------------------------------------------------------------------------------
--                                                                          --
--                         GNAT RUN-TIME COMPONENTS                         --
--                                                                          --
--                       S Y S T E M . T E X T _ I O                        --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--          Copyright (C) 1992-2015, Free Software Foundation, Inc.         --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 3,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.                                     --
--                                                                          --
--                                                                          --
--                                                                          --
--                                                                          --
--                                                                          --
-- You should have received a copy of the GNU General Public License and    --
-- a copy of the GCC Runtime Library Exception along with this program;     --
-- see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    --
-- <http://www.gnu.org/licenses/>.                                          --
--                                                                          --
-- GNAT was originally developed  by the GNAT team at  New York University. --
-- Extensive contributions were provided by Ada Core Technologies Inc.      --
--                                                                          --
------------------------------------------------------------------------------

------------------------------------------------------------------------------
--  Minimal version of Text_IO body for use on TM4C1294 using UART0         --
------------------------------------------------------------------------------

with TM4C1294;
with TM4C1294.GPIO; use TM4C1294.GPIO;

package body System.Text_IO is

   UART_Base : constant := 16#4000_C000#;

   UARTDR   : TM4C1294.Word with Volatile,
              Address => System'To_Address (UART_Base + 16#000#);
   UARTFR   : TM4C1294.Word with Volatile,
              Address => System'To_Address (UART_Base + 16#018#);
   UARTIBRD : TM4C1294.Word with Volatile,
              Address => System'To_Address (UART_Base + 16#024#);
   UARTFBRD : TM4C1294.Word with Volatile,
              Address => System'To_Address (UART_Base + 16#028#);
   UARTLCRH : TM4C1294.Word with Volatile,
              Address => System'To_Address (UART_Base + 16#02C#);
   UARTCTL  : TM4C1294.Word with Volatile,
              Address => System'To_Address (UART_Base + 16#030#);

   procedure Set_Baud_Rate (Baud_Rate : Positive);

------------------------------------------------------------------------------
--  Get                                                                     --
------------------------------------------------------------------------------

   function Get return Character is
      use type TM4C1294.Word;
   begin
      return Character'Val (UARTDR and 16#FF#);
   end Get;

------------------------------------------------------------------------------
--  Set_Baud_Rate                                                           --
------------------------------------------------------------------------------

   procedure Set_Baud_Rate (Baud_Rate : Positive) is
      Clk_Div  : constant Natural := 16;
      BRD      : Natural;
      FBRD     : Natural;
      Clk_Freq : constant Positive := 120_000_000;
   begin

      BRD :=  Clk_Freq / (Clk_Div * (Baud_Rate / 100));

      UARTIBRD := TM4C1294.Word (BRD / 100);

      FBRD := BRD mod 100;

      FBRD := (63 * FBRD) / 99;

      UARTFBRD := TM4C1294.Word (FBRD);

   end Set_Baud_Rate;

------------------------------------------------------------------------------
--  Initialize                                                              --
------------------------------------------------------------------------------

   procedure Initialize is
      Welcome : constant String := "TM4C1294 GNAT GPL 2015" &
                                    ASCII.CR & ASCII.LF;
      use type TM4C1294.Word;
   begin
      Initialized := True;

      TM4C1294.Enable_UART_Clocking (TM4C1294.UART0);
      TM4C1294.Enable_Port_Clocking (TM4C1294.PortA);

      --  Configure IO ports ...
      --  U0RX=PA0, U0TX=PA1, U0CTS=PB4, U0RTS=PB5

      GPIO_Set_Mode (TM4C1294.PortA, 0, Alt_Function, 1);
      GPIO_Set_Mode (TM4C1294.PortA, 1, Alt_Function, 1);

      --  Configure UART ...

      --  Disable UART ...
      UARTCTL := UARTCTL and 16#FFFF_FFFE#;

      --  Set 8 bits no parity ...
      UARTLCRH := (UARTLCRH and 16#FFFF_FF00#) or 2#0111_1000#;

      Set_Baud_Rate (115_200);

      UARTCTL := UARTCTL or 16#0000_0001#;

      for w in Welcome'Range loop
         while not Is_Tx_Ready loop
            null;
         end loop;
         Put (Welcome (w));
      end loop;

   end Initialize;

------------------------------------------------------------------------------
--  Is_Tx_Ready                                                             --
------------------------------------------------------------------------------

   function Is_Tx_Ready return Boolean is
      use type TM4C1294.Word;
   begin
      return (UARTFR and TM4C1294.BIT5) = 0;
   end Is_Tx_Ready;

------------------------------------------------------------------------------
--  Is_Rx_Ready                                                             --
------------------------------------------------------------------------------

   function Is_Rx_Ready return Boolean is
      use type TM4C1294.Word;
   begin
      return (UARTFR and TM4C1294.BIT4) = 0;
   end Is_Rx_Ready;

------------------------------------------------------------------------------
--  Put                                                                     --
------------------------------------------------------------------------------

   procedure Put (C : Character) is
   begin
      UARTDR := TM4C1294.Word (Character'Pos (C));
   end Put;

------------------------------------------------------------------------------
--  Use_Cr_Lf_For_New_Line                                                  --
------------------------------------------------------------------------------

   function Use_Cr_Lf_For_New_Line return Boolean is (True);

end System.Text_IO;
