------------------------------------------------------------------------------
--                                                                          --
--                         GNAT RUN-TIME COMPONENTS                         --
--                                                                          --
--          Copyright (C) 2012-2015, Free Software Foundation, Inc.         --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 3,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.                                     --
--                                                                          --
--                                                                          --
--                                                                          --
--                                                                          --
--                                                                          --
-- You should have received a copy of the GNU General Public License and    --
-- a copy of the GCC Runtime Library Exception along with this program;     --
-- see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    --
-- <http://www.gnu.org/licenses/>.                                          --
--                                                                          --
-- GNAT was originally developed  by the GNAT team at  New York University. --
-- Extensive contributions were provided by Ada Core Technologies Inc.      --
--                                                                          --
------------------------------------------------------------------------------

--  TM4C1294 Connected Launchpad board
--  With 25MHz external crystal, 120MHz system clock

pragma Ada_2012; -- To work around pre-commit check?
pragma Restrictions (No_Elaboration_Code);

with System;
with TM4C1294; use TM4C1294;
with TM4C1294.GPIO; use TM4C1294.GPIO;

procedure Setup_Pll is

   --  Use PLL ...
   SYSCTL_RSCLKCFG_USEPLL : constant Word := 16#1000_0000#;
   --  Power Down ...
   SYSCTL_MOSCCTL_PWRDN : constant Word := 16#0000_0008#;
   --  No Crystal Connected ...
   SYSCTL_MOSCCTL_NOXTAL : constant Word := 16#0000_0004#;
   --  Oscillator Source Mask ...
   SYSCTL_RSCLKCFG_OSCSRC_M : constant Word := 16#00F0_0000#;
   --  MOSC is oscillator source ...
   SYSCTL_RSCLKCFG_OSCSRC_MOSC : constant Word := 16#0030_0000#;
   --  MOSC Power Up Raw Interrupt Status ...
   SYSCTL_RIS_MOSCPUPRIS : constant Word := 16#0000_0100#;
   --  PLL Source Mask ...
   SYSCTL_RSCLKCFG_PLLSRC_M : constant Word := 16#0F00_0000#;
   --  MOSC is the PLL input clock source ...
   SYSCTL_RSCLKCFG_PLLSRC_MOSC : constant Word := 16#0300_0000#;
   --  Memory Timing Register Update ...
   SYSCTL_RSCLKCFG_MEMTIMU : constant Word := 16#8000_0000#;
   --  PLL System Clock Divisor ..
   SYSCTL_RSCLKCFG_PSYSDIV_M : constant Word := 16#0000_03FF#;
   --  PLL lock ...
   SYSCTL_PLLSTAT_LOCK : constant Word := 16#0000_0001#;
   --  PSYSDIV for 120MHz clock from 480MHz PLL ...
   PSYSDIV : constant Word := 3;
   --  Deep Sleep Oscillator Source Mask ...
   SYSCTL_DSCLKCFG_DSOSCSRC_M : constant Word := 16#00F0_0000#;
   --  Deep Sleep Oscillator Source ...
   SYSCTL_DSCLKCFG_DSOSCSRC_MOSC : constant Word := 16#0030_0000#;
   --  PLL M Fractional Value
   SYSCTL_PLLFREQ0_MFRAC_M : constant Word := 16#000F_FC00#;
   --  PLL M Integer Value
   SYSCTL_PLLFREQ0_MINT_M  : constant Word := 16#0000_03FF#;
   --  PLL Power
   SYSCTL_PLLFREQ0_PLLPWR : constant := 16#0080_0000#;
   --  New PLLFREQ Accept
   SYSCTL_RSCLKCFG_NEWFREQ : constant := 16#4000_0000#;

   SYSCTL_PLLFREQ0_MFRAC_S : constant := BIT10;
   SYSCTL_PLLFREQ0_MINT_S : constant := BIT0;

   --  PLL Q Value
   SYSCTL_PLLFREQ1_Q_M : constant Word := 16#0000_1F00#;
   --  PLL N Value
   SYSCTL_PLLFREQ1_N_M : constant Word := 16#0000_001F#;

   SYSCTL_PLLFREQ1_Q_S : constant := BIT8;
   SYSCTL_PLLFREQ1_N_S : constant := BIT0;

   --  25MHz external crystal ...
   FXTAL : constant := 25_000_000;
   Q : constant := 0;
   N : constant := 4;
   --  480,000,000 = (25,000,000/(0 + 1)/(4 + 1))*(96 + (0/1,024))
   MINT : constant := 96;
   --  zero to reduce jitter
   MFRAC : constant := 0;

   procedure Initialize_Clocks;
   procedure Reset_Clocks;

   use type Word;

------------------------------------------------------------------------------
-- Initialize_Clocks                                                        --
------------------------------------------------------------------------------

   procedure Initialize_Clocks is
      SYSCTL : System_Control_Registers with Volatile,
                  Address => System'To_Address (Sys_Ctrl_Base_Address);
      Timeout : Natural;
      SYSCLK  : Positive;
      pragma Unreferenced (SYSCLK);
   begin

      SYSCLK := (FXTAL / (Q + 1) / (N + 1)) *
                (MINT + MFRAC / 1024) / (Positive (PSYSDIV) + 1);

      SYSCTL.PLLFREQ0 := (SYSCTL.PLLFREQ0 and not SYSCTL_PLLFREQ0_MFRAC_M) or
                         (MFRAC * SYSCTL_PLLFREQ0_MFRAC_S) or
                         (SYSCTL.PLLFREQ0 and not SYSCTL_PLLFREQ0_MINT_M) or
                         (MINT * SYSCTL_PLLFREQ0_MINT_S);

      SYSCTL.PLLFREQ1 := (SYSCTL.PLLFREQ1 and not SYSCTL_PLLFREQ1_Q_M) or
                         (Q * SYSCTL_PLLFREQ1_Q_S) or
                         (SYSCTL.PLLFREQ1 and not SYSCTL_PLLFREQ1_N_M) or
                         (N * SYSCTL_PLLFREQ1_N_S);

      --  Turn on power to PLL
      SYSCTL.PLLFREQ0 := SYSCTL.PLLFREQ0 or SYSCTL_PLLFREQ0_PLLPWR;

      --  Lock in register changes ...
      SYSCTL.RSCLKCFG :=  SYSCTL.RSCLKCFG or SYSCTL_RSCLKCFG_NEWFREQ;

      --  Configure FLASH settings for 120MHz SYSCLK...

      --  FBCHT/EBCHT = 6, FBCE/EBCE = 0, FWS/EWS = 5
      SYSCTL.MEMTIM0 := (SYSCTL.MEMTIM0 and not 16#03EF_03EF#) or
                        (6 * BIT22) + (0 * BIT21) + (5 * BIT16) +
                        (6 * BIT6) + (0 * BIT5) + 5;

      --  PLL clock from main oscillator ...
      SYSCTL.RSCLKCFG := (SYSCTL.RSCLKCFG and not SYSCTL_RSCLKCFG_PLLSRC_M) or
                            SYSCTL_RSCLKCFG_PLLSRC_MOSC;

      Timeout := 0;
      loop
         exit when (SYSCTL.PLLSTAT and SYSCTL_PLLSTAT_LOCK) /= 0;
         Timeout := Timeout + 1;
         exit when Timeout >= 16#FFFF#;
      end loop;

      if Timeout >= 16#FFFF# then
         Blink_LED2;
         return;
      end if;

      SYSCTL.RSCLKCFG := (SYSCTL.RSCLKCFG and not SYSCTL_RSCLKCFG_PSYSDIV_M) or
                            (PSYSDIV and SYSCTL_RSCLKCFG_PSYSDIV_M) or
                            SYSCTL_RSCLKCFG_MEMTIMU or SYSCTL_RSCLKCFG_USEPLL;
   end Initialize_Clocks;

------------------------------------------------------------------------------
-- Reset_Clocks                                                             --
------------------------------------------------------------------------------

   procedure Reset_Clocks is
      SYSCTL : System_Control_Registers with Volatile,
                  Address => System'To_Address (Sys_Ctrl_Base_Address);
   begin
      --  Before setting up PLL make sure it is not clock source ...
      SYSCTL.RSCLKCFG := SYSCTL.RSCLKCFG and not SYSCTL_RSCLKCFG_USEPLL;

      --  Enable external crystal and turn off power down ...
      SYSCTL.MOSCCTL := SYSCTL.MOSCCTL and not
                           (SYSCTL_MOSCCTL_NOXTAL or SYSCTL_MOSCCTL_PWRDN);

      loop
         --  Wait for the MOSCPUPRIS bit to be set in the SYSCTL_RIS_R
         --   register, indicating that MOSC crystal mode is ready.
         exit when (SYSCTL.RIS and SYSCTL_RIS_MOSCPUPRIS) /= 0;
      end loop;

      --  Temporarily get run/sleep clock from 25 MHz main oscillator ...
      SYSCTL.RSCLKCFG := (SYSCTL.RSCLKCFG and not SYSCTL_RSCLKCFG_OSCSRC_M) or
                            SYSCTL_RSCLKCFG_OSCSRC_MOSC;

      --  PLL clock from main oscillator.
      SYSCTL.RSCLKCFG := (SYSCTL.RSCLKCFG and not SYSCTL_RSCLKCFG_PLLSRC_M) or
                            SYSCTL_RSCLKCFG_PLLSRC_MOSC;

      SYSCTL.DSCLKCFG := (SYSCTL.DSCLKCFG and not SYSCTL_DSCLKCFG_DSOSCSRC_M)
                          or SYSCTL_DSCLKCFG_DSOSCSRC_MOSC;

   end Reset_Clocks;

begin
   Reset_Clocks;
   Initialize_Clocks;
end Setup_Pll;
