-----------------------------------------------------------------------------
--  Ti Tiva TM4C1294 UART driver                                           --
-----------------------------------------------------------------------------

with TM4C1294.GPIO; use TM4C1294.GPIO;

package body TM4C1294.UART is

   use type Word;

   FR_REG_CTS  : constant Word := BIT0;
   FR_REG_DSR  : constant Word := BIT1;
   FR_REG_DCD  : constant Word := BIT2;
   FR_REG_BUSY : constant Word := BIT3;
   FR_REG_RXFE : constant Word := BIT4;
   FR_REG_TXFF : constant Word := BIT5;
   FR_REG_RXFF : constant Word := BIT6;
   FR_REG_RI   : constant Word := BIT7;

   pragma Unreferenced (FR_REG_CTS, FR_REG_DSR, FR_REG_DCD);
   pragma Unreferenced (FR_REG_BUSY, FR_REG_RXFF, FR_REG_RI);

   UARTCTL_RXE    : constant Word := BIT9;
   UARTCTL_TXE    : constant Word := BIT8;
   UARTCTL_UARTEN : constant Word := BIT0;

   UARTLCRH_PEN : constant Word := BIT1;
   UARTLCRH_EPS_EVEN : constant Word := BIT2;
   UARTLCRH_WLEN_MASK : constant Word := BIT5 + BIT6;
   UARTLCRH_STP2 : constant Word := BIT3;

-----------------------------------------------------------------------------
--  UART_Enable                                                            --
-----------------------------------------------------------------------------

   procedure UART_Enable (UART : in out UART_IO) is
   begin
      UART.Regs.UARTCTL := UART.Regs.UARTCTL or UARTCTL_RXE or UARTCTL_TXE or
                           UARTCTL_UARTEN;
   end UART_Enable;

-----------------------------------------------------------------------------
--  UART_Disable                                                           --
-----------------------------------------------------------------------------

   procedure UART_Disable (UART : in out UART_IO) is
   begin
      UART.Regs.UARTCTL := UART.Regs.UARTCTL and not UARTCTL_UARTEN;
   end UART_Disable;

-----------------------------------------------------------------------------
--  UART_Tx_Ready                                                          --
-----------------------------------------------------------------------------

   function UART_Tx_Ready (UART : UART_IO) return Boolean is
   begin
      return (UART.Regs.UARTFR and FR_REG_TXFF) = 0;
   end UART_Tx_Ready;

-----------------------------------------------------------------------------
--  UART_Rx_Ready                                                          --
-----------------------------------------------------------------------------

   function UART_Rx_Ready (UART : UART_IO) return Boolean is
   begin
      return (UART.Regs.UARTFR and FR_REG_RXFE) = 0;
   end UART_Rx_Ready;

-----------------------------------------------------------------------------
--  UART_Get                                                               --
-----------------------------------------------------------------------------

   function UART_Get (UART : UART_IO) return Character is
   begin
      return Character'Val (UART.Regs.UARTDR and 16#FF#);
   end UART_Get;

-----------------------------------------------------------------------------
--  UART_Put                                                               --
-----------------------------------------------------------------------------

   procedure UART_Put (UART : in out UART_IO; C : Character) is
      Ch : constant Word := Word (Character'Pos (C));
   begin
      UART.Regs.UARTDR := ((UART.Regs.UARTDR) and 16#FFFF_FF00#) or Ch;
   end UART_Put;

-----------------------------------------------------------------------------
--  UART_Put                                                               --
-----------------------------------------------------------------------------

   procedure UART_Put (UART : in out UART_IO; S : String) is
   begin
      for n in S'Range loop
         UART_Put (UART, S (n));
      end loop;
   end UART_Put;

------------------------------------------------------------------------------
--  UART_Configure                                                          --
------------------------------------------------------------------------------

   procedure UART_Configure (UART      : in out UART_IO;
                             Data_Bits : UART_Data_Bits := 8;
                             Stop_Bits : UART_Stop_Bits := One;
                             Parity_On : Boolean := False;
                             Parity    : UART_Parity := Even) is
   begin

      if Parity_On then
         UART.Regs.UARTLCRH := UART.Regs.UARTLCRH or UARTLCRH_PEN;
         if Parity = Even then
            UART.Regs.UARTLCRH := UART.Regs.UARTLCRH or UARTLCRH_EPS_EVEN;
         else
            UART.Regs.UARTLCRH := UART.Regs.UARTLCRH and not UARTLCRH_EPS_EVEN;
         end if;
      else
         UART.Regs.UARTLCRH := UART.Regs.UARTLCRH and not UARTLCRH_PEN;
      end if;

      Blink_LED1;

      if Stop_Bits = One then
         UART.Regs.UARTLCRH := UART.Regs.UARTLCRH and not UARTLCRH_STP2;
      else
         UART.Regs.UARTLCRH := UART.Regs.UARTLCRH or UARTLCRH_STP2;
      end if;

      Blink_LED1;

      case Data_Bits is
         when 5 =>
            UART.Regs.UARTLCRH := (UART.Regs.UARTLCRH and not
                                     UARTLCRH_WLEN_MASK) or 2#000_0000#;
         when 6 =>
            UART.Regs.UARTLCRH := (UART.Regs.UARTLCRH and not
                                     UARTLCRH_WLEN_MASK) or 2#010_0000#;
         when 7 =>
            UART.Regs.UARTLCRH := (UART.Regs.UARTLCRH and not
                                     UARTLCRH_WLEN_MASK) or 2#100_0000#;
         when 8 =>
            UART.Regs.UARTLCRH := (UART.Regs.UARTLCRH and not
                                     UARTLCRH_WLEN_MASK) or 2#110_0000#;
      end case;

   end UART_Configure;

------------------------------------------------------------------------------
--  UART_Enable_FIFO                                                        --
------------------------------------------------------------------------------

   procedure UART_Enable_FIFO (UART : in out UART_IO) is
      FEN : constant Word := BIT4;
   begin
      UART.Regs.UARTLCRH := UART.Regs.UARTLCRH or FEN;
   end UART_Enable_FIFO;

------------------------------------------------------------------------------
--  UART_Set_Baud_Rate                                                      --
------------------------------------------------------------------------------

   procedure UART_Set_Baud_Rate (UART        : in out UART_IO;
                                 Baud_Rate   : Positive;
                                 Use_Sys_Clk : Boolean := False;
                                 Clk_Freq    : Positive := 16_000_000) is
      HSE     : constant := BIT5;
      Clk_Div : Natural;
      BRD     : Natural;
      FBRD    : Natural;
   begin

      if Use_Sys_Clk then
         UART.Regs.UARTCC := UART.Regs.UARTCC and 16#FFF0#;
      else
         UART.Regs.UARTCC := (UART.Regs.UARTCC and 16#FFF0#) or 5;
      end if;

      if (UART.Regs.UARTCTL and HSE) = 0 then
         Clk_Div := 16;
      else
         Clk_Div := 8;
      end if;

      BRD := Clk_Freq / (Clk_Div * (Baud_Rate / 100));

      UART.Regs.UARTIBRD := Word (BRD / 100);

      FBRD := BRD mod 100;

      FBRD := (63 * FBRD) / 99;

      UART.Regs.UARTFBRD := Word (FBRD);

   end UART_Set_Baud_Rate;

end TM4C1294.UART;
